@@ -240,7 +240,12 @@ static char *resolve_path(const char *path, int follow_link)
 		else
 			ret = realpath(path, filtered_path);
 
-		/* Maybe we failed because of funky anonymous fd symlinks.
+		/* Handle broken symlinks.  This can come up for a variety of reasons,
+		 * but we need to make sure that we resolve the path all the way to the
+		 * final target, and not just where the current link happens to start.
+		 * Latest discussion is in #540828.
+		 *
+		 * Maybe we failed because of funky anonymous fd symlinks.
 		 * You can see this by doing something like:
 		 *		$ echo | ls -l /proc/self/fd/
 		 *		.......	0 -> pipe:[9422999]
@@ -248,11 +253,19 @@ static char *resolve_path(const char *path, int follow_link)
 		 * actual file paths for us to check against. #288863
 		 * Don't look for any particular string as these are dynamic
 		 * according to the kernel.  You can see pipe:, socket:, etc...
+		 *
+		 * Maybe we failed because it's a symlink to a path in /proc/ that
+		 * is a symlink to a path that longer exists -- readlink will set
+		 * ENOENT even in that case and the file ends in (deleted).  This
+		 * can come up in cases like:
+		 * /dev/stderr -> fd/2 -> /proc/self/fd/2 -> /removed/file (deleted)
 		 */
-		if (!ret && !strncmp(filtered_path, "/proc/", 6)) {
-			char *base = strrchr(filtered_path, '/');
-			if (base && strchr(base, ':'))
-				ret = filtered_path;
+		if (!ret && errno == ENOENT) {
+			ret = canonicalize_filename_mode(path, CAN_ALL_BUT_LAST);
+			if (ret) {
+				free(filtered_path);
+				filtered_path = ret;
+			}
 		}
 
 		if (!ret) {
@@ -306,7 +319,16 @@ static char *resolve_path(const char *path, int follow_link)
 char *egetcwd(char *buf, size_t size)
 {
 	struct stat st;
-	char *tmpbuf, *oldbuf = buf;
+	char *tmpbuf;
+
+	/* We can't let the C lib allocate memory for us since we have our
+	 * own local routines to handle things.
+	 */
+	bool allocated = (buf == NULL);
+	if (allocated) {
+		size = SB_PATH_MAX;
+		buf = xmalloc(size);
+	}
 
 	/* If tracing a child, our cwd may not be the same as the child's */
 	if (trace_pid) {
@@ -354,9 +376,9 @@ char *egetcwd(char *buf, size_t size)
 		  	errno = ENAMETOOLONG;
 
 		if (errno && errno != EACCES) {
-		  	/* If getcwd() allocated the buffer, free it. */
-			if (NULL == oldbuf)
-				free(tmpbuf);
+			/* If getcwd() allocated the buffer, free it. */
+			if (allocated)
+				free(buf);
 
 			/* Not sure if we should quit here, but I guess if
 			 * lstat() fails, getcwd could have messed up. Not
@@ -368,6 +390,9 @@ char *egetcwd(char *buf, size_t size)
 
 		restore_errno();
 	} else if (errno != 0) {
+		/* If getcwd() allocated the buffer, free it. */
+		if (allocated)
+			free(buf);
 
 		/* Make sure we do not return garbage if the current libc or
 		 * kernel's getcwd() is buggy.
@@ -887,6 +912,8 @@ static int check_syscall(sbcontext_t *sbcontext, int sb_nr, const char *func,
 	resolved_path = resolve_path(file, 1);
 	if (!absolute_path || !resolved_path)
 		goto error;
+	sb_debug_dyn("absolute_path: %s\n", absolute_path);
+	sb_debug_dyn("resolved_path: %s\n", resolved_path);
 
 	verbose = is_env_set_on(ENV_SANDBOX_VERBOSE, &set);
 	if (set)
@@ -1073,9 +1100,9 @@ bool before_syscall_open_char(int dirfd, int sb_nr, const char *func, const char
 	if ((*mode == 'r') && ((0 == (strcmp(mode, "r"))) ||
 	     /* The strspn accept args are known non-writable modifiers */
 	     (strlen(mode+1) == strspn(mode+1, "xbtmce"))))
-		sb_nr = SB_NR_OPEN_RD, ext_func = "open_rd";
+		sb_nr = SB_NR_OPEN_RD, ext_func = "fopen_rd";
 	else
-		sb_nr = SB_NR_OPEN_WR, ext_func = "open_wr";
+		sb_nr = SB_NR_OPEN_WR, ext_func = "fopen_wr";
 	return before_syscall(dirfd, sb_nr, ext_func, file, 0);
 }
 
