@@ -59,6 +59,11 @@ static long _do_ptrace(enum __ptrace_request request, const char *srequest, void
 			}
 			sched_yield();
 			goto try_again;
+		} else if (errno == EIO || errno == EFAULT) {
+			/* This comes up when the child itself tries to use a bad pointer.
+			 * That's not something the sandbox should abort on. #560396
+			 */
+			return ret;
 		} else if (!errno)
 			if (request == PTRACE_PEEKDATA ||
 			    request == PTRACE_PEEKTEXT ||
@@ -93,10 +98,14 @@ static char *do_peekstr(unsigned long lptr)
 #ifdef HAVE_PROCESS_VM_READV
 	struct iovec liov, riov;
 
-	/* We can't cross remote page boundaries when using this :( */
+	/* We can't cross remote page boundaries when using this :(.
+	 * The first call reads up to the edge of the page boundary since we
+	 * have no (easy) way of knowing if the next page is valid.  This is
+	 * less effort than reading in the /proc/.../maps file and walking it.
+	 */
 	l = 0x1000;
 	riov.iov_base = (void *)lptr;
-	len = lptr % l;
+	len = l - (lptr % l);
 	if (!len)
 		len = l;
 	liov.iov_base = ret = xmalloc(len);
@@ -105,28 +114,23 @@ static char *do_peekstr(unsigned long lptr)
 	while (1) {
 		if (process_vm_readv(trace_pid, &liov, 1, &riov, 1, 0) == -1) {
 			int e = errno;
-			if (e == ENOSYS)
+			if (e == ENOSYS) {
+				/* This can happen if run on older kernels but built with newer ones. */
 				break;
-			sb_eqawarn("process_vm_readv(%i, %p{%p, %zu}, 1, %p{%p, %zu}, 1, 0) failed: %s\n",
+			} else if (e == EFAULT) {
+				/* This can happen if the target process uses a bad pointer. #560396 */
+				break;
+			}
+			sb_ebort("ISE:do_peekstr:process_vm_readv(%i, %p{%p, %#zx}, 1, %p{%p, %#zx}, 1, 0) failed: %s\n",
 				trace_pid,
 				&liov, liov.iov_base, liov.iov_len,
 				&riov, riov.iov_base, riov.iov_len,
 				strerror(e));
-			if (e == EFAULT) {
-				/* This sometimes happens, but it's not clear why.
-				 * Throw some extended debugging info before falling
-				 * back to the ptrace code. #560396
-				 */
-				break;
-			}
-			sb_ebort("ISE:do_peekstr: process_vm_readv() hates us: %s\n",
-				strerror(e));
 		}
 
-		for (i = 0; i < liov.iov_len; ++i)
-			if (!((char *)liov.iov_base)[i])
-				return ret;
-		riov.iov_base += l;
+		if (memchr(liov.iov_base, '\0', liov.iov_len) != NULL)
+			return ret;
+		riov.iov_base += riov.iov_len;
 		riov.iov_len = liov.iov_len = l;
 		len += l;
 		ret = xrealloc(ret, len);
@@ -141,7 +145,16 @@ static char *do_peekstr(unsigned long lptr)
 	while (1) {
 		a = lptr & (sizeof(long) - 1);
 		lptr -= a;
+		errno = 0;
 		s.val = do_peekdata(lptr);
+		if (unlikely(errno)) {
+			if (errno == EIO || errno == EFAULT) {
+				ret[0] = '\0';
+				return ret;
+			}
+			sb_ebort("ISE:do_peekstr:do_peekdata(%#lx) failed: %s\n",
+				lptr, strerror(errno));
+		}
 		for (i = a; i < sizeof(long); ++i) {
 			ret[l++] = s.x[i];
 			if (!s.x[i])
