@@ -1,7 +1,4 @@
-static int trace_sysnum(void)
-{
-	return do_peekuser(PT_R15);
-}
+#define trace_sysnum_puser PT_R15
 
 static long trace_raw_ret(void *vregs)
 {
